# AlignmentParView, creates view on alignments (without data) on .axt files (BlastZ) and .xmft (Vista)
#
# Parameter lines look like:
# .axt:  (alignment number) (chromosome) (start) (end) (2nd chrom) (2nd start) (2nd end) (2nd strand) (score)

import dbview,filez


class AlignmentParView(dbview.View):

    # NOTE: Module name and class name should be identical (see View._getInstanceCode)
    
    def createView(self, paramDict, parentViews, viewDB, id):
        super(AlignmentParView,self).createView(paramDict, parentViews, viewDB, id)
        if len(parentViews)==1:
            # NetView-backed
            self.net = parentViews[0]
            self.havenet = True
            return
        self.havenet = False
        # load actual data
        self.alist = []
        filename = paramDict['filename']
        handle = filez.open(filename,'r')
        eof = 0
        while not eof:
            while 1:
                elts = handle.readline()[:-1].split(' ')
                if len(elts)==0 or len(elts[0])==0 or elts[0][0] != '#':
                    break
            if len(elts) < 9:
                eof = 1
            else:
                filepos = handle.tell()
                handle.readline()
                handle.readline()
                handle.readline()
                (start1,end1,start2,end2) = long(elts[2]), long(elts[3]), long(elts[5]), long(elts[6])
                # in .axt file, counts start from 1, and segments are inclusive
                start1 -= 1
                start2 -= 1
                self.alist.append( [ start1, end1, start2, end2, elts[1], elts[4], elts[7], float(elts[8]), filepos ] )
        handle.close()
        # now cache them
        self.cache(self.alist,'alist')


    def loadView(self, paramDict, parentViews, viewDB, id):
        super(AlignmentParView,self).loadView(paramDict, parentViews, viewDB, id)
        if len(parentViews)==1:
            # NetView-backed
            self.net = parentViews[0]
            self.havenet = True
            return
        self.havenet = False
        self.alist = self.uncache('alist')


    def init(self):
        super(AlignmentParView,self).init()
        self.fields = {'start1':0,'end1':1,'start2':2,'end2':3,'chr1':4,'chr2':5,'strand2':6,'score':7,'filepos':8,'strand1':9}


    def iter(self):
        return self.alist.iter()


    def __getitem__(self,key):
        if self.havenet:
            ntup = self.net[key]
            return (ntup[1], ntup[2],ntup[4],ntup[5],ntup[0],ntup[3],ntup[6],6000,key,'+')
        tup = self.alist[key]
        return (tup[0],tup[1],tup[2],tup[3],tup[4],tup[5],tup[6],tup[7],tup[8],'+')


    def __len__(self):
        if self.havenet:
            return self.net.__len__()
        return self.alist.__len__()
