# AlignmentView, creates view on alignments from .axt files, based on AlignmentParView

from __future__ import generators


import dbview,filez


class AlignmentView(dbview.View):

    # NOTE: Module name and class name should be identical (see View._getInstanceCode)
    
    def createView(self, paramDict, parentViews, viewDB, id):
        super(AlignmentView,self).createView(paramDict, parentViews, viewDB, id)
        # no data to load, this is purely a view


    def loadView(self, paramDict, parentViews, viewDB, id):
        super(AlignmentView,self).loadView(paramDict, parentViews, viewDB, id)


    def init(self):
        super(AlignmentView,self).init()
        filename = self.parentViews[0].paramDict['filename']
        if type(filename) != type([]):
            filename = [filename]
        self.file = [filez.open(file,'r') for file in filename]
        if self.parentViews[0].name == "AlignmentParView" or self.parentViews[0].name == "AlignmentParView2":
            self.apv = 1
            self.fileposlabel = self.parentViews[0].fields['filepos']
        elif self.parentViews[0].name == "MAFParView":
            self.apv = 2
            self.fileposlabel = self.parentViews[0].fields['filepos']
        else:
            self.apv = 3
            assert self.parentViews[0].name == "MAFXParView"
        

    def __del__(self):
        for f in self.file:
            f.close()


    def __iter__(self):
        self._idx = 0
        return self


    def next(self):
        try:
            data = self.__getitem__(self._idx)
            self._idx += 1
            return data
        except:
            raise StopIteration


    def __getitem__(self,key):
        if self.apv == 1:   # AlignmentParView
            filepos = self.parentViews[0].alist[key][self.fileposlabel]
            self.file[0].seek(filepos)
            return [self.file[0].readline()[:-1],self.file[0].readline()[:-1]]
        if self.apv == 2:  # MAFParView
            filepos = self.parentViews[0].alist[key][self.fileposlabel]
            data = [0 for i in range(len(self.parentViews[0].labels))]
            f = self.parentViews[0].alist[key][0]
            self.file[f].seek(filepos)
            line = self.file[f].readline()
            labels = self.parentViews[0].labels
            while len(line)>2:
                (es,specchrom,pos,nuclen,strand,totlen,seq) = line.split()
                (spec,chrom) = specchrom.split('.')
                if labels.has_key( spec ):
                    data[labels[spec]] = seq
                line = self.file[f].readline()
            return data
        if self.apv == 3:  # MAFXParView
            # Make empty list for all of the species
            numspecies = len(self.parentViews[0].labels)
            data = ["" for i in range(numspecies)]
            file = self.file[0]
            # Get parameters
            pars = self.parentViews[0].alist[key]
            # Get file positions
            fps = pars[ 5 ]
            # Get alignments.  Loop over all blocks
            for i in range(len(fps[0])):
                # Loop over all species
                alignlen = -1
                for j in range(numspecies):
                    if fps[j][i] != -1:
                        # data exists
                        file.seek( fps[j][i] )
                        if self.parentViews[0].type == 'maf':
                            (es,specchrom,pos,nuclen,strand,totlen,seq) = file.readline().split()
                        else:
                            seqs = []
                            line = file.readline()[:-1]
                            while line[0] != '=' and line[0] != '>':
                                seqs.append(line[:-1])
                                line = file.readline()[:-1]
                            seq = "".join(seqs)
                        # add 'missing' base sequence, for all but last block
                        if i < len(fps[0])-1:
                            glue = pars[ 0 ][ i ]
                            if j != self.parentViews[0].base:
                                # non-base sequence: add dashes
                                glue = ''.join(['-' for k in glue])
                        else:
                            glue = ""
                        seq += glue
                        data[j] += seq
                        alignlen = len(seq)
                # Now alignlen is initialized; complete this block
                for j in range(numspecies):
                    if fps[j][i] == -1:
                        data[j] += ''.join(['-' for k in range(alignlen)])
            return data
        assert False


    def __len__(self):
        return self.parentViews[0].alist.__len__()


