# GeneListView, creates view on genes in ensGene.txt file (and similar ones), for single chromosome or entire genome


import dbview


class GeneListView(dbview.View):

    # NOTE: Module name and class name should be identical (see View._getInstanceCode)
    # NOTE!!: This works for hg16, not for hg15!
    
    def createView(self, paramDict, parentViews, viewDB, id):
        super(GeneListView,self).createView(paramDict, parentViews, viewDB, id)
        # load actual data
        self.genes = []
        filename = paramDict['filename']
        chromosome = paramDict['chromosome']
        handle = open(filename,'r')
        for line in handle:
            elts = line.split('\t')
            if elts[1] == chromosome or chromosome == '':
                exonstart = eval('['+elts[8][:-1]+']')    # skip '# of exons' field
                # next line: used to be [:-1] to remove trailing ',\n'; now there is no ',' anymore (hg16)
                exonend = eval('['+elts[9][:-1]+']')      # remove trailing ',' and ',\n'
                # next line: used to subtract 1 from start
                exons = [(exonstart[i],exonend[i]) for i in range(len(exonstart))]
                # next line: used to subtract 1 from start
                introns = [(exonend[i],exonstart[i+1]) for i in range(len(exonstart)-1)]
                self.genes.append( [ elts[1], \
                                     elts[2], \
                                     # used to subtract 1 from next
                                     long(elts[3]), \
                                     long(elts[4]), \
                                     # used to subtract 1 from next
                                     long(elts[5]), \
                                     long(elts[6]),
                                     exons,
                                     introns,
                                     elts[0] ] )
        handle.close()
        # now cache them
        self.cache(self.genes,'genes')


    def loadView(self, paramDict, parentViews, viewDB, id):
        super(GeneListView,self).loadView(paramDict, parentViews, viewDB, id)
        self.genes = self.uncache('genes')


    def init(self):
        super(GeneListView,self).init()
        self.fields = {'chromosome':0,'dir':1,'start':2,'end':3,'codingstart':4,'codingend':5,'exons':6,'introns':7,'name':8}


    def iter(self):
        return self.genes.iter()


    def __getitem__(self,key):
        return self.genes[key]


    def __len__(self):
        return self.genes.__len__()
