# RepeatView, creates view on repeat element database in .fa.out files


# Addition: if 'teonly'==1 exists, takes only elements where class is 'LINE', 'SINE', 'LTR' or 'DNA'.
# Addition: extra field 'classroot', LINE SINE etc.

import dbview,filez


class RepeatView(dbview.View):

    # NOTE: Module name and class name should be identical (see View._getInstanceCode)
    
    def createView(self, paramDict, parentViews, viewDB, id):
        super(RepeatView,self).createView(paramDict, parentViews, viewDB, id)
        # load actual data
        self.replist = []
        idoffset = 0
        maxid = 0
        filename = paramDict['filename']
        if paramDict.has_key('teonly'):
            self.teonly = paramDict['teonly']
        else:
            self.teonly = 0
        if paramDict.has_key('keepall'):
            self.keepall = 1
        else:
            self.keepall = 0
        handle = filez.open(filename,'r')
        handle.readline()
        handle.readline()
        handle.readline()
        lines = handle.readlines()
        handle.close()
        for line in lines:
            elts = line[:-1].split(None)
            if len(elts) == 15:
                if elts[8] == '+':
                    begidx = 11
                else:
                    begidx = 13
                id = int(elts[14]) + idoffset
                ## heuristic to get unique identifiers
                if id < maxid-30 and id-idoffset == 1:
                    idoffset = maxid
                    id = int(elts[14]) + idoffset
                if self.teonly==0 or ((elts[10]+"   ")[:3] in ['LIN','SIN','LTR','DNA']):
                    te = (  int(elts[0]), \
                            int(10.0*float(elts[1])+0.5), \
                            int(10.0*float(elts[2])+0.5), \
                            int(10.0*float(elts[3])+0.5), \
                            elts[4], \
                            long(elts[5])-1, \
                            long(elts[6]), \
                            elts[8], \
                            elts[9], \
                            elts[10], \
                            int(elts[begidx])-1, \
                            int(elts[12]), \
                            int(elts[24-begidx][1:-1]) + int(elts[12]), \
                            id,
                            (elts[10].split('/'))[0] )
                    if self.keepall:
                        self.replist.append( te )
                    else:
                        self.replist.append( (te[4],te[5],te[6],te[7],te[8],te[9],te[13],te[14]) )
                maxid = max(id,maxid)
            else:
                break
        # now cache them
        self.cache(self.replist,'replist')


    def loadView(self, paramDict, parentViews, viewDB, id):
        super(RepeatView,self).loadView(paramDict, parentViews, viewDB, id)
        self.replist = self.uncache('replist')


    def init(self):
        super(RepeatView,self).init()
        if self.paramDict.has_key('keepall'):
            self.keepall = 1
        else:
            self.keepall = 0
        if self.keepall:
            self.fields = {'swscore':0,'percdiv':1,'percdel':2,'percins':3,'chrom':4,'start':5,'end':6,'strand':7,\
                           'repeat':8,'class':9,'repstart':10,'repend':11,'replen':12,'id':13,'classroot':14}
        else:
            self.fields = {'chrom':0,'start':1,'end':2,'strand':3,'repeat':4,'class':5,'id':6,'classroot':7}


    def iter(self):
        return self.replist.iter()


    def __getitem__(self,key):
        return self.replist[key]


    def __len__(self):
        return self.replist.__len__()
