# SequenceSelectView, creates view of segments, based on criterion operating on sequence view
# (Is used to filter out N nucleotides)

# parameters:
# 'criterion': function taking sequence data, yielding list of segments to be included
#              This must be a string that evaluates to a function (to ensure that the database
#              itself does not require the function to be present)
# 'window': window size (default: 10000)
# 'dx': stride (default: window size)

from __future__ import generators


import dbview
import Seg



class SequenceSelectView(dbview.View):

    # NOTE: Module name and class name should be identical (see View._getInstanceCode)
    
    def createView(self, paramDict, parentViews, viewDB, id):
        super(SequenceSelectView,self).createView(paramDict, parentViews, viewDB, id)
        self.segList = []
        curseg = (0,self.window)
        try:
            while curseg[0] < len(parentViews[0]):
                seq = parentViews[0][curseg[0]:curseg[1]]
                segs = self.criterion(seq)
                segs = [Seg.plus(seg,curseg[0]) for seg in segs]
                self.segList.extend(segs)
                curseg = Seg.plus(curseg,self.dx)
        except IndexError:
            pass
        self.segList.sort()
        self.segList = [seg for seg in self.segList if not Seg.empty(seg)]
        # now remove overlaps, and represent list as efficiently as possible
        for i in range(1,len(self.segList)):
            if Seg.touch(self.segList[i-1],self.segList[i]):
                self.segList[i] = Seg.union( self.segList[i-1],self.segList[i] )
                self.segList[i-1] = Seg.nullSeg
        # remove empties
        self.segList = [seg for seg in self.segList if not Seg.empty(seg)]
        # now invert if asked
        self._invert()
        self.cache(self.segList,'seglist')


    def loadView(self, paramDict, parentViews, viewDB, id):
        super(SequenceSelectView,self).loadView(paramDict, parentViews, viewDB, id)
        self.segList = self.uncache('seglist')


    def init(self):
        super(SequenceSelectView,self).init()
        self.criterion = eval( self.paramDict['criterion'], self.globals )
        if self.paramDict.has_key('window'):
            self.window = self.paramDict['window']
        else:
            self.window = 1000
        if self.paramDict.has_key('dx'):
            self.dx = self.paramDict['dx']
        else:
            self.dx = self.window


    def setGlobals(self, globals):
        self.globals = globals


    def __iter__(self):
        return iter(self.segList)


    def __getitem__(self,key):
        return self.segList[key]


    def __len__(self):
        return len(self.segList)


