# TabListView, generates view on tab-delimited column lists
# Needs parameter 'columns', list of keys, at least one of which should be 'key' or 666 signifying a key,
# others should be numeric value for numbers, '' for strings, or [numeric] or [''] for lists of numbers or string
# (non-unique per key).  The numeric value is used for values that are absent.  Multiple keys are combined.
#
# Or, parameter 'keys', list of form ['key',pos,type,'key',pos,type], where 'type' is an integer, real or string.
# If these are not 0, 0.0 or '' resp, will only include lines where value is equal
#



import dbview,filez
import re,array


class SameArray(object):

    def __init__(self, value):
        self.value = value
        self.len = 0

    def append(self,ob):
        self.len += 1

    def __getitem__(self,idx):
        return self.value

    def __setitem__(self,idx,obj):
        self.len = idx+1

    def __len__(self):
        return self.len
    

class TabListView(dbview.View):

    def cacheList(self, list):
        list2 = list[:]
        for i in range(len(list2)):
            if type(list2[i]) == array.array:
                #filename = self.getFileName("tablistarray") + "."+str(i)
                #file = open(filename,"wb")
                #list2[i].tofile( file )
                #file.close()
                filename = "array"+str(i)
                aslist = list2[i].tolist()
                self.cache( aslist, filename )
                del aslist
                list2[i] = "ARRAY" + list[i].typecode
        self.cache( list2, "tablist" )


    def uncacheList(self):
        list2 = self.uncache( "tablist" )
        for i in range(len(list2)):
            if type(list2[i]) == str and list2[i] and list2[i][:-1] == "ARRAY":
                #filename = self.getFileName("tablistarray") + "."+str(i)
                #file = open(filename,"rb")
                list2[i] = array.array( list2[i][-1] )
                #try:
                #    list2[i].fromfile( file, 999999999 )
                #except EOFError:
                #    pass
                #file.close()
                filename = "array"+str(i)
                aslist = self.uncache( filename )
                list2[i].fromlist( aslist )
                del aslist
        return list2

    # NOTE: Module name and class name should be identical (see View._getInstanceCode)
    
    def createView(self, paramDict, parentViews, viewDB, id):
        super(TabListView,self).createView(paramDict, parentViews, viewDB, id)
        # load actual data
        filename = paramDict['filename']
        try:
            handle = filez.open(filename,'r')
        except IOError:
            print "Warning: file ",filename," seems not to exist.  Proceeding with empty tab list."
            handle = []
        if paramDict.has_key('skiplines'):
            skiplines = paramDict['skiplines']   # >0: skip this many lines; <0: skip blanks and # lines
        else:
            skiplines = 0
        if paramDict.has_key('delimiter'):
            delimiter = paramDict['delimiter']
        else:
            delimiter = '\t'
        if paramDict.has_key('numcols'):
            numcols = paramDict['numcols']       # ignore lines with different number of columns
        else:
            numcols = -1
        if paramDict.has_key('commas'):
            commas = paramDict['commas']
        else:
            commas = False
        lineno = 0
        if paramDict.has_key('keys'):
            self.tablist = []
            keys = paramDict['keys']
            k = [keys[i] for i in range(0,len(keys),3)]
            p = [keys[i] for i in range(1,len(keys),3)]
            t = [keys[i] for i in range(2,len(keys),3)]        # type, and selection value
            s = [t[i]!=0 and t[i]!="" for i in range(len(t))]  # which fields to select
            # build empty tablist
            for i in range(len(t)):
                if type(t[i]) == long or type(t[i]) == int:
                    if s[i]:
                        self.tablist.append( SameArray( t[i] ) )
                        #print "Same ",i
                    else:
                        self.tablist.append( array.array('l') )
                        #print "Array ",i
                elif type(t[i]) == float:
                    if s[i]:
                        self.tablist.append( SameArray( t[i] ) )
                        #print "Same ",i
                    else:
                        self.tablist.append( array.array('d') )
                        #print "Array ",i
                elif type(t[i]) == str:
                    t[i] = t[i].strip()
                    if s[i]:
                        self.tablist.append( SameArray( t[i] ) )
                        #print "Same ",i,t[i]
                    else:
                        self.tablist.append( [] )
                        #print "List ",i
                else:
                    print "Unrecognized type: ",t[i]
                    sys.exit(1)
            if paramDict.has_key('select'):
                s = paramDict['select']
            for line in handle:
                lineno += 1
                if lineno % 100000 == 0:
                    print "TabListView: now at line ",lineno
                if skiplines < 0 and line[0] != '\n' and line[0]!='#':
                    skiplines = 0
                if skiplines != 0:
                    skiplines -= 1
                else:
                    if delimiter == '\t':
                        raw = line[:-1].split('\t')
                    else:
                        # add a leading delimiter, to avoid leading delimiters (spaces) shift all columns by one
                        raw = re.sub("\\"+delimiter+"+",delimiter,delimiter+line[:-1]).split(delimiter)[1:]
                    take = True
                    data = []
                    numcommas = -1
                    try:
                        if numcols>-1 and len(raw) != numcols:
                            raise(RuntimeError())
                        for i in range(len(k)):
                            if type(t[i]) == int or type(t[i]) == long:
                                if s[i] or not commas:
                                    data.append( long( raw[ p[i] ] ) )
                                else:
                                    data.append( map(long, raw[ p[i] ].split(',')[:-1] ) )
                                    if numcommas != -1 and numcommas != len(data[-1]):
                                        print "TabListView: multiple comma-delimited columns, but unequal number of elements at line",lineno
                                        print raw
                                        sys.exit(1)
                                    numcommas = len( data[-1] )
                                if s[i]:
                                    if data[-1] != t[i]:
                                        take = False
                                        break
                            elif type(t[i]) == float:
                                data.append( float( raw[ p[i] ] ) )
                                if s[i]:
                                    if data[-1] != t[i]:              # this is stupid... but consistent
                                        take = False
                                        break
                            elif type(t[i]) == str:
                                data.append( intern( raw[ p[i] ].strip() ) )
                                if s[i]:
                                    if data[-1] != t[i]:
                                        take = False
                                        break
                            else:
                                print "Wrong type: ",t[i]
                    except:
                        print "Couldn't interpret line:",raw," line=",lineno
                    if take:
                        if numcommas >= 0:
                            for j in range(numcommas):
                                for i in range(len(data)):
                                    if type(data[i]) == type([]):
                                        self.tablist[i].append( data[i][j] )
                                    else:
                                        self.tablist[i].append( data[i] )
                        else:
                            #self.tablist.append( data )
                            for i in range(len(data)):
                                self.tablist[i].append( data[i] )
            self.fields = {}
            for i in range(len(k)):
                self.fields[ k[i] ] = i
            #print "Found ",len(self.tablist[0])," lines"
        else:
            # not supported - configuration of the tablist has changed to accomodate arrays
            print "Provide keys!"
            sys.exit(1)
            columns = paramDict['columns']
            defaults = []
            colcodes = []
            havekey = 0
            for i in range(len(columns)):
                colc = 0
                col = columns[i]
                if col == 666 or col == 'key':
                    havekey = 1
                    colc = 4
                if type(col) == type([]):
                    # non-unique
                    colc |= 2
                    col = col[0]
                defaults.append(col)
                if type(col) == type(''):
                    # string
                    colc |= 1
                colcodes.append(colc)
            if havekey == 0:
                print "Need at least one column denoted key ('column' value 666 or  'key')"
            keymap = {}
            self.tablist = []
            for line in handle:
                lineno += 1
                if lineno % 10000 == 0:
                    print "TabListView: Now at line ",lineno
                if skiplines != 0:
                    skiplines -= 1
                else:
                    if delimiter == '\t':
                        elts = line[:-1].split('\t')
                    else:
                        elts = re.sub("\\"+delimiter+"+",delimiter,line[:-1]).split(" ")
                    key = ''
                    for i in range(len(columns)):
                        #print i,elts[i],columns[i]
                        if colcodes[i] & 1 == 0:
                            # convert string rep of number to number
                            if elts[i] != '':
                                elts[i] = long(elts[i])
                            else:
                                elts[i] = defaults[i]
                        if colcodes[i] & 4:
                            key += '*'+str(elts[i])
                        if colcodes[i] & 2:
                            elts[i] = [elts[i]]
                    if keymap.has_key(key):
                        idx = keymap[key]
                        # add non-unique elements if needed
                        for i in range(len(columns)):
                            if colcodes[i] & 2:
                                if not elts[i][0] in self.tablist[idx][i]:
                                    self.tablist[idx][i].append(elts[i][0])
                            else:
                                if not elts[i] == self.tablist[idx][i]:
                                    print "Warning: Adding record:\n",elts,"\nto record:\n",self.tablist[idx],\
                                          "\nand found non-identical column ",i," which should be unique."
                    else:
                        idx = len(self.tablist)
                        keymap[key] = idx
                        self.tablist.append(elts)
            del keymap
        try:
            handle.close()
        except AttributeError:
            # if file does not exist, handle is a list.  Continue silently
            pass
        # now cache them
        if id != -1:
            self.cacheList( self.tablist )


    def loadView(self, paramDict, parentViews, viewDB, id):
        super(TabListView,self).loadView(paramDict, parentViews, viewDB, id)
        keys = paramDict['keys']
        k = [keys[i] for i in range(0,len(keys),3)]
        for i in range(len(k)):
            self.fields[ k[i] ] = i
        self.tablist = self.uncacheList()
        # try to build it from scratch if we get a MmeoryError?


    def init(self):
        super(TabListView,self).init()
        self.fields = {}


    def __iter__(self):
        self._idx = -1
        return self


    def next(self):
        self._idx += 1
        if self._idx < len(self.tablist[0]):
            return self.__getitem__(self._idx)
        raise StopIteration


    def __getitem__(self,key):
        return map( lambda x: x[key], self.tablist )


    def __len__(self):
        return self.tablist[0].__len__()

