"""
Seg - tools for dealing with half-open segments [a,b), represented by 2-tuples (a,b)
"""

import sys


nullSeg = (0,0)
allSeg = (-sys.maxint,sys.maxint)


def empty(s):
    return s[0]>=s[1]


def len(s):
    # returns length of segment, 0 for empty
    if s[0]>=s[1]:
        return 0
    return s[1]-s[0]


def overlap(s,t):
    # returns true if s and t share elements
    return max(s[0],t[0]) < min(s[1],t[1])


def overlapelt(s,n):
    # returns true if s and t share elements
    return s[0] <= n < s[1]


def distance(s,x):
    # returns distance between segment s and point or segment x
    if empty(s):
        return sys.maxint;
    if type(x)==type(nullSeg) or type(x)==type([0]):
        if empty(x):
            return sys.maxint
        if touch(s,x):
            return 0
        return max(s[0]-x[1], x[0]-s[1])
    else:
        if overlapelt(s,x):
            return 0
        return max(s[0]-x, x-s[1])


def touch(s,t):
    # returns true if s,t are nonempty, and union of s and t is a single segment
    return max(s[0],t[0]) <= min(s[1],t[1])


def intersection(s,t):
    # returns intersection of segments
    return (max(s[0],t[0]),min(s[1],t[1]))


def inside(s,t):
    # returns true if s is inside t
    return s[0]>=t[0] and t[1]>=s[1] and not empty(t)


def unionL(s,t):
    # returns union of segments, or leftmost segment if non-intersecting
    if empty(s):
        return nullSeg
    if empty(t):
        return s
    if s[0]<t[0]:
        if s[1]<=t[0]:
            return s
        else:
            return (s[0],t[1])
    else:
        if t[1]<=s[0]:
            return t
        else:
            return (t[0],s[1])


def unionR(s,t):
    # returns union of segments, or rightmost segment if non-intersecting
    if empty(s):
        return t
    if empty(t):
        return nullSeg
    if s[0]<t[0]:
        if s[1]<=t[0]:
            return t
        else:
            return nullSeg
    else:
        if t[1]<=s[0]:
            return s
        else:
            return nullSeg

def union(s,t):
    # returns union including any intermediate segment
    if empty(s):
        return t
    if empty(t):
        return s
    return (min(s[0],t[0]), max(s[1],t[1]))



def minusL(s,t):
    # returns s minus t, left part
    if empty(t):
        return s
    if empty(s):
        return s
    if s[0]<t[0]:
        return (s[0],min(t[0],s[1]))
    return nullSeg


def minusR(s,t):
    if empty(t):
        return s
    if empty(s):
        return s
    if t[1]<s[1]:
        return (max(s[0],t[1]),s[1])
    return nullSeg


def plus(s,dx):
    return (s[0]+dx,s[1]+dx)


def union(s,t):
    # returns union of segments, including middle part if any
    if empty(s):
        return t
    if empty(t):
        return s
    return (min(s[0],t[0]),max(s[1],t[1]))


def sort(slist):
    # returns sorted list of segments
    def _compare(s,t):
        if empty(s):
            return -1
        if empty(t):
            return 1
        if s[0]<t[0]:
            return -1
        if s[0]>t[0]:
            return 1
        return cmp(s[1],t[1])
    slist.sort(_compare)
        
    
