#
# mygetopt - interpret command line options
#


def getopt(cmdline, data, dict = {}, parsepart = False):
    # Usage: getopt(sys.argv[1:], [("-size=",GetSize,"Size of file")])
    #
    # The GetSize callback gets called with the option string "-size=" and the remainder.
    # It should return an error string if there is an error, False or "" if not
    #
    # parsepart = False: getopt returns False if there was an error; the list of remaining cmd line args if not ([""] if none)
    # parsepart = True: getopt returns True if there was an error; False if not

    setdefaults(data, dict)
    for i in range(len(cmdline)):
        c = cmdline[i]
        if c == '--' and parsepart:
            if len(cmdline) == i:
                return [""]
            else:
                return cmdline[i+1:]
        processed = False
        for d in data:
            # see if we have a synonym
            org = d[0].split(',')[0]
            syns = d[0].split(',')[1:]
            for s in syns:
                if (c+("*"*len(s)))[:len(s)] == s:
                    c = org+c[len(s):]
            if (c+("*"*len(org)))[:len(org)] == org:
                optdata = c[len(org):]
                if d[1]==getstring or d[1]==getint or d[1]==getfloat:
                    error = d[1](org,optdata,dict)
                else:
                    error = d[1](org,optdata)
                if error:
                    print "Error in command line option '"+c+"': ",error
                    printhelp(data)
                    return parsepart
                processed = True
        if not processed:
            print "Command line option '"+c+"' not recognized"
            printhelp(data)
            return parsepart

    if parsepart:
        return [""]
    else:
        return True


def printhelp(options):
    print
    print "Available options are:"
    for opt in options:
        (optstring,func,help) = (opt[0],opt[1],opt[2])
        if len(opt)==4:
            default = " (default:%s)" % opt[3]
        else:
            default = ""
        print "%15s : %s%s" % (optstring,help,default)
    print

            
def _test(o,s):
    print "Received option data ",o,"=",s
    if s == "err":
        return "Error handling works"
    return ""


def getstring( opt, data, settings ):
    id = opt[1:-1]                # strip the "-" and "=" from the option string
    if id not in settings:
        print "Option '"+id+"' not recognized"
        print "Available options: ",settings.keys()
        sys.exit(1)
    settings[id] = data


def getint( opt, data, settings ):
    id = opt[1:-1]                # strip the "-" and "=" from the option string
    if id not in settings:
        print "Option '"+id+"' not recognized"
        print "Available options: ",settings.keys()
        sys.exit(1)
    settings[id] = int(data)


def getfloat( opt, data, settings ):
    id = opt[1:-1]                # strip the "-" and "=" from the option string
    if id not in settings:
        print "Option '"+id+"' not recognized"
        print "Available options: ",settings.keys()
        sys.exit(1)
    settings[id] = float(data)



def setdefaults( options, settings ):
    for opt in options:
        id = opt[0][1:-1]         # strip the "-" and "=" from the option string
        settings[ id ] = opt[-1]  # set default



#getopt(["-opt=ok","-opt=err"],[("-opt=",_test,"Option help text")])


